@@ -15,7 +15,7 @@ from distributed.deploy.utils_test import ClusterTest
 
 def test_simple(loop):
     with LocalCluster(4, scheduler_port=0, nanny=False, silence_logs=False,
-            loop=loop) as c:
+            diagnostic_port=None, loop=loop) as c:
         with Executor((c.scheduler.ip, c.scheduler.port), loop=loop) as e:
             x = e.submit(inc, 1)
             x.result()
@@ -26,7 +26,7 @@ def test_simple(loop):
 @pytest.mark.skipif('sys.version_info[0] == 2', reason='multi-loop')
 def test_procs(loop):
     with LocalCluster(2, scheduler_port=0, nanny=False, threads_per_worker=3,
-            silence_logs=False) as c:
+            diagnostic_port=None, silence_logs=False) as c:
         assert len(c.workers) == 2
         assert all(isinstance(w, Worker) for w in c.workers)
         with Executor((c.scheduler.ip, c.scheduler.port), loop=loop) as e:
@@ -34,7 +34,7 @@ def test_procs(loop):
         repr(c)
 
     with LocalCluster(2, scheduler_port=0, nanny=True, threads_per_worker=3,
-            silence_logs=False) as c:
+            diagnostic_port=None, silence_logs=False) as c:
         assert len(c.workers) == 2
         assert all(isinstance(w, Nanny) for w in c.workers)
         with Executor((c.scheduler.ip, c.scheduler.port), loop=loop) as e:
@@ -47,59 +47,77 @@ def test_procs(loop):
 
 @pytest.mark.skipif('sys.version_info[0] == 2', reason='')
 class LocalTest(ClusterTest, unittest.TestCase):
-    Cluster = partial(LocalCluster, silence_logs=False)
+    Cluster = partial(LocalCluster, silence_logs=False, diagnostic_port=None)
 
 
 def test_Executor_with_local(loop):
-    with LocalCluster(1, scheduler_port=0, silence_logs=False, loop=loop) as c:
+    with LocalCluster(1, scheduler_port=0, silence_logs=False,
+            diagnostic_port=None, loop=loop) as c:
         with Executor(c, loop=loop) as e:
             assert len(e.ncores()) == len(c.workers)
             assert c.scheduler_address in repr(e)
 
 
 def test_Executor_solo(loop):
-    e = Executor(loop=loop)
-    e.shutdown()
+    with Executor(loop=loop) as e:
+        pass
+    assert e.cluster.status == 'closed'
+
+
+def test_Executor_twice(loop):
+    with Executor(loop=loop) as e:
+        with Executor(loop=loop) as f:
+            assert e.cluster.scheduler.port != f.cluster.scheduler.port
 
 
 def test_defaults():
     from distributed.worker import _ncores
 
-    with LocalCluster(scheduler_port=0, silence_logs=False) as c:
+    with LocalCluster(scheduler_port=0, silence_logs=False,
+            diagnostic_port=None) as c:
         assert sum(w.ncores for w in c.workers) == _ncores
         assert all(isinstance(w, Nanny) for w in c.workers)
         assert all(w.ncores == 1 for w in c.workers)
 
-    with LocalCluster(nanny=False, scheduler_port=0, silence_logs=False) as c:
+    with LocalCluster(nanny=False, scheduler_port=0, silence_logs=False,
+            diagnostic_port=None) as c:
         assert sum(w.ncores for w in c.workers) == _ncores
         assert all(isinstance(w, Worker) for w in c.workers)
         assert len(c.workers) == 1
 
-    with LocalCluster(n_workers=2, scheduler_port=0, silence_logs=False) as c:
+    with LocalCluster(n_workers=2, scheduler_port=0, silence_logs=False,
+            diagnostic_port=None) as c:
         assert sum(w.ncores for w in c.workers) == max(2, _ncores)
 
-    with LocalCluster(threads_per_worker=_ncores * 2, scheduler_port=0, silence_logs=False) as c:
+    with LocalCluster(threads_per_worker=_ncores * 2, scheduler_port=0,
+            silence_logs=False, diagnostic_port=None) as c:
         assert len(c.workers) == 1
 
-    with LocalCluster(n_workers=_ncores * 2, scheduler_port=0, silence_logs=False) as c:
+    with LocalCluster(n_workers=_ncores * 2, scheduler_port=0,
+            silence_logs=False, diagnostic_port=None) as c:
         assert all(w.ncores == 1 for w in c.workers)
-    with LocalCluster(threads_per_worker=2, n_workers=3, scheduler_port=0, silence_logs=False) as c:
+    with LocalCluster(threads_per_worker=2, n_workers=3, scheduler_port=0,
+            silence_logs=False, diagnostic_port=None) as c:
         assert len(c.workers) == 3
         assert all(w.ncores == 2 for w in c.workers)
 
 
 def test_cleanup():
-    c = LocalCluster(2, scheduler_port=0, silence_logs=False)
+    c = LocalCluster(2, scheduler_port=0, silence_logs=False,
+            diagnostic_port=None)
     port = c.scheduler.port
     c.close()
-    c2 = LocalCluster(2, scheduler_port=port, silence_logs=False)
+    c2 = LocalCluster(2, scheduler_port=port, silence_logs=False,
+            diagnostic_port=None)
     c.close()
 
 
 def test_repeated():
-    with LocalCluster(scheduler_port=8448, silence_logs=False) as c:
+    with LocalCluster(scheduler_port=8448, silence_logs=False,
+            diagnostic_port=None) as c:
         pass
-    with LocalCluster(scheduler_port=8448, silence_logs=False) as c:
+    with LocalCluster(scheduler_port=8448, silence_logs=False,
+            diagnostic_port=None) as c:
         pass
 
 
@@ -107,7 +125,8 @@ def test_http(loop):
     from distributed.http import HTTPScheduler
     import requests
     with LocalCluster(scheduler_port=0, silence_logs=False,
-            services={('http', 3485): HTTPScheduler}, loop=loop) as c:
+            services={('http', 3485): HTTPScheduler}, diagnostic_port=None,
+            loop=loop) as c:
         response = requests.get('http://127.0.0.1:3485/info.json')
         assert response.ok
 
@@ -116,7 +135,8 @@ def test_bokeh(loop):
     from distributed.http import HTTPScheduler
     import requests
     with LocalCluster(scheduler_port=0, silence_logs=False, loop=loop,
-            diagnostics_port=4724, services={('http', 0): HTTPScheduler}) as c:
+            diagnostics_port=4724, services={('http', 0): HTTPScheduler},
+            diagnostic_port=None) as c:
         start = time()
         while True:
             with ignoring(Exception):
@@ -136,7 +156,8 @@ def test_bokeh(loop):
 def test_start_diagnostics(loop):
     from distributed.http import HTTPScheduler
     import requests
-    with LocalCluster(scheduler_port=0, silence_logs=False, loop=loop) as c:
+    with LocalCluster(scheduler_port=0, silence_logs=False, loop=loop,
+            diagnostics_port=None) as c:
         c.start_diagnostics_server(show=False, port=3748)
 
         start = time()
