@@ -7,6 +7,7 @@ from datetime import timedelta
 from functools import partial
 import logging
 import os
+import sys
 from time import sleep
 import uuid
 from threading import Thread
@@ -436,7 +437,9 @@ class Executor(object):
                 for msg in msgs:
                     logger.debug("Executor receives message %s", msg)
 
-                    if msg['op'] == 'close':
+                    if msg.get('status') == 'scheduler-error':
+                        six.reraise(*clean_exception(**msg))
+                    elif msg['op'] == 'close':
                         breakout = True
                         break
                     elif msg['op'] == 'key-in-memory':
@@ -571,7 +574,9 @@ class Executor(object):
             else:
                 key = funcname(func) + '-' + str(uuid.uuid4())
 
-        if tokey(key) in self.futures:
+        skey = tokey(key)
+
+        if skey in self.futures:
             return Future(key, self)
 
         if allow_other_workers and workers is None:
@@ -580,23 +585,23 @@ class Executor(object):
         if isinstance(workers, str):
             workers = [workers]
         if workers is not None:
-            restrictions = {key: workers}
-            loose_restrictions = [key] if allow_other_workers else []
+            restrictions = {skey: workers}
+            loose_restrictions = [skey] if allow_other_workers else []
         else:
             restrictions = {}
             loose_restrictions = []
 
         if kwargs:
-            dsk = {key: (apply, func, list(args), kwargs)}
+            dsk = {skey: (apply, func, list(args), kwargs)}
         else:
-            dsk = {key: (func,) + tuple(args)}
+            dsk = {skey: (func,) + tuple(args)}
 
-        futures = self._graph_to_futures(dsk, [key], restrictions,
-                loose_restrictions, priority={key: 0})
+        futures = self._graph_to_futures(dsk, [skey], restrictions,
+                loose_restrictions, priority={skey: 0})
 
         logger.debug("Submit %s(...), %s", funcname(func), key)
 
-        return futures[tokey(key)]
+        return futures[skey]
 
     def _threaded_map(self, q_out, func, qs_in, **kwargs):
         """ Internal function for mapping Queue """
@@ -950,7 +955,7 @@ class Executor(object):
                         broadcast=broadcast)
     @gen.coroutine
     def _cancel(self, futures):
-        keys = {f.key for f in futures_of(futures)}
+        keys = {tokey(f.key) for f in futures_of(futures)}
         yield self.scheduler.cancel(keys=list(keys), client=self.id)
         for k in keys:
             with ignoring(KeyError):
@@ -1122,7 +1127,7 @@ class Executor(object):
         results2 = pack_data(keys, results)
         return results2
 
-    def compute(self, args, sync=False, optimize_graph=True):
+    def compute(self, args, sync=False, optimize_graph=True, **kwargs):
         """ Compute dask collections on cluster
 
         Parameters
@@ -1131,6 +1136,10 @@ class Executor(object):
             Collections like dask.array or dataframe or dask.value objects
         sync: bool (optional)
             Returns Futures if False (default) or concrete values if True
+        optimize_graph: bool
+            Whether or not to optimize the underlying graphs
+        kwargs:
+            Options to pass to the graph optimize calls
 
         Returns
         -------
@@ -1169,7 +1178,7 @@ class Executor(object):
         if optimize_graph:
             groups = groupby(lambda x: x._optimize, variables)
             dsk = merge([opt(merge([v.dask for v in val]),
-                             [v._keys() for v in val])
+                             [v._keys() for v in val], **kwargs)
                         for opt, val in groups.items()])
         else:
             dsk = merge(c.dask for c in variables)
@@ -1198,7 +1207,7 @@ class Executor(object):
         else:
             return result
 
-    def persist(self, collections, optimize_graph=True):
+    def persist(self, collections, optimize_graph=True, **kwargs):
         """ Persist dask collections on cluster
 
         Starts computation of the collection on the cluster in the background.
@@ -1209,6 +1218,10 @@ class Executor(object):
         ----------
         collections: sequence or single dask object
             Collections like dask.array or dataframe or dask.value objects
+        optimize_graph: bool
+            Whether or not to optimize the underlying graphs
+        kwargs:
+            Options to pass to the graph optimize calls
 
         Returns
         -------
@@ -1234,7 +1247,7 @@ class Executor(object):
         if optimize_graph:
             groups = groupby(lambda x: x._optimize, collections)
             dsk = merge([opt(merge([v.dask for v in val]),
-                             [v._keys() for v in val])
+                             [v._keys() for v in val], **kwargs)
                         for opt, val in groups.items()])
         else:
             dsk = merge(c.dask for c in collections)
@@ -1567,11 +1580,32 @@ class Executor(object):
         return sync(self.loop, self.scheduler.nbytes, keys=keys,
                     summary=summary)
 
+    def scheduler_info(self):
+        """ Basic information about the workers in the cluster
+
+        Examples
+        --------
+        >>> e.scheduler_info()  # doctest: +SKIP
+        {'id': '2de2b6da-69ee-11e6-ab6a-e82aea155996',
+         'services': {},
+         'type': 'Scheduler',
+         'workers': {'127.0.0.1:40575': {'active': 0,
+                                         'last-seen': 1472038237.4845693,
+                                         'name': '127.0.0.1:40575',
+                                         'services': {},
+                                         'stored': 0,
+                                         'time-delay': 0.0061032772064208984}}}
+        """
+        return sync(self.loop, self.scheduler.identity)
+
     def futures_of(self, futures):
         return futures_of(futures, executor=self)
 
+    def start_ipython(self, *args, **kwargs):
+        raise Exception("Method moved to start_ipython_workers")
+
     @gen.coroutine
-    def _start_ipython(self, workers):
+    def _start_ipython_workers(self, workers):
         if workers is None:
             workers = yield self.scheduler.ncores()
 
@@ -1580,7 +1614,8 @@ class Executor(object):
         )
         raise gen.Return((workers, responses))
 
-    def start_ipython(self, workers=None, magic_names=False, qtconsole=False, qtconsole_args=None):
+    def start_ipython_workers(self, workers=None, magic_names=False,
+                              qtconsole=False, qtconsole_args=None):
         """ Start IPython kernels on workers
 
         Parameters
@@ -1595,11 +1630,30 @@ class Executor(object):
         qtconsole: bool (optional)
             If True, launch a Jupyter QtConsole connected to the worker(s).
 
+        qtconsole_args: list(str) (optional)
+            Additional arguments to pass to the qtconsole on startup.
+
+        Examples
+        --------
+        >>> info = e.start_ipython_workers() # doctest: +SKIP
+        >>> %remote info['192.168.1.101:5752'] worker.data  # doctest: +SKIP
+        {'x': 1, 'y': 100}
+
+        >>> e.start_ipython_workers('192.168.1.101:5752', magic_names='w') # doctest: +SKIP
+        >>> %w worker.data  # doctest: +SKIP
+        {'x': 1, 'y': 100}
+
+        >>> e.start_ipython_workers('192.168.1.101:5752', qtconsole=True) # doctest: +SKIP
+
         Returns
         -------
         iter_connection_info: list
             List of connection_info dicts containing info necessary
             to connect Jupyter clients to the workers.
+
+        See Also
+        --------
+        Executor.start_ipython_scheduler: start ipython on the scheduler
         """
 
         if magic_names and isinstance(magic_names, six.string_types):
@@ -1607,7 +1661,11 @@ class Executor(object):
         if isinstance(workers, six.string_types):
             workers = [workers]
 
-        (workers, info_dict) = sync(self.loop, self._start_ipython, workers)
+        (workers, info_dict) = sync(self.loop, self._start_ipython_workers, workers)
+
+        if 'IPython' in sys.modules:
+            from ._ipython_utils import register_remote_magic
+            register_remote_magic()
         if magic_names:
             from ._ipython_utils import register_worker_magic
             for worker, magic_name in zip(workers, magic_names):
@@ -1622,6 +1680,62 @@ class Executor(object):
                 )
         return info_dict
 
+    def start_ipython_scheduler(self, magic_name='scheduler_if_ipython',
+                                qtconsole=False, qtconsole_args=None):
+        """ Start IPython kernel on the scheduler
+
+        Parameters
+        ----------
+        magic_name: str or None (optional)
+            If defined, register IPython magic with this name for
+            executing code on the scheduler.
+            If not defined, register %scheduler magic if IPython is running.
+
+        qtconsole: bool (optional)
+            If True, launch a Jupyter QtConsole connected to the worker(s).
+
+        qtconsole_args: list(str) (optional)
+            Additional arguments to pass to the qtconsole on startup.
+
+        Examples
+        --------
+        >>> e.start_ipython_scheduler() # doctest: +SKIP
+        >>> %scheduler scheduler.processing  # doctest: +SKIP
+        {'127.0.0.1:3595': {'inc-1', 'inc-2'},
+         '127.0.0.1:53589': {'inc-2', 'add-5'}}
+
+        >>> e.start_ipython_scheduler(qtconsole=True) # doctest: +SKIP
+
+        Returns
+        -------
+        connection_info: dict
+            connection_info dict containing info necessary
+            to connect Jupyter clients to the scheduler.
+
+        See Also
+        --------
+        Executor.start_ipython_workers: Start IPython on the workers
+        """
+        info = sync(self.loop, self.scheduler.start_ipython)
+        if magic_name == 'scheduler_if_ipython':
+            # default to %scheduler if in IPython, no magic otherwise
+            in_ipython = False
+            if 'IPython' in sys.modules:
+                from IPython import get_ipython
+                in_ipython = bool(get_ipython())
+            if in_ipython:
+                magic_name = 'scheduler'
+            else:
+                magic_name = None
+        if magic_name:
+            from ._ipython_utils import register_worker_magic
+            register_worker_magic(info, magic_name)
+        if qtconsole:
+            from ._ipython_utils import connect_qtconsole
+            connect_qtconsole(info, name='dask-scheduler',
+                              extra_args=qtconsole_args,)
+        return info
+
 
 class CompatibleExecutor(Executor):
     """ A concurrent.futures-compatible Executor
@@ -1654,7 +1768,11 @@ def _wait(fs, timeout=None, return_when='ALL_COMPLETED'):
     if timeout is not None:
         raise NotImplementedError("Timeouts not yet supported")
     if return_when == 'ALL_COMPLETED':
-        yield All({f.event.wait() for f in fs})
+        try:
+            yield All({f.event.wait() for f in fs})
+        except KeyError:
+            raise CancelledError([f.key for f in fs
+                                        if f.key not in f.executor.futures])
         done, not_done = set(fs), set()
     else:
         raise NotImplementedError("Only return_when='ALL_COMPLETED' supported")
