@@ -7,7 +7,7 @@ from concurrent.futures import CancelledError
 from datetime import timedelta
 import itertools
 from multiprocessing import Process
-from random import random
+from random import random, choice
 import sys
 from threading import Thread
 from time import sleep, time
@@ -16,7 +16,7 @@ import traceback
 import mock
 import pytest
 from toolz import (identity, isdistinct, first, concat, pluck, valmap,
-        partition_all, partial)
+        partition_all, partial, sliding_window)
 from tornado import gen
 from tornado.ioloop import IOLoop
 from tornado.iostream import StreamClosedError
@@ -236,19 +236,6 @@ def test_sync_exceptions(loop):
         e.shutdown()
 
 
-@gen_cluster(executor=True)
-def test_stress_1(e, s, a, b):
-    n = 2**6
-
-    seq = e.map(inc, range(n))
-    while len(seq) > 1:
-        yield gen.sleep(0.1)
-        seq = [e.submit(add, seq[i], seq[i + 1])
-                for i in range(0, len(seq), 2)]
-    result = yield seq[0]._result()
-    assert result == sum(map(inc, range(n)))
-
-
 @gen_cluster(executor=True)
 def test_gather(e, s, a, b):
     x = e.submit(inc, 10)
@@ -446,17 +433,6 @@ def test_recompute_released_key(e, s, a, b):
     assert result1 == result2
 
 
-@pytest.mark.parametrize(('func', 'n'), [(slowinc, 100), (inc, 1000)])
-def test_stress_gc(loop, func, n):
-    with cluster() as (s, [a, b]):
-        with Executor(('127.0.0.1', s['port']), loop=loop) as e:
-            x = e.submit(func, 1)
-            for i in range(n):
-                x = e.submit(func, x)
-
-            assert x.result() == n + 2
-
-
 @slow
 @gen_cluster(executor=True)
 def test_long_tasks_dont_trigger_timeout(e, s, a, b):
@@ -642,45 +618,6 @@ def test_remove_worker(e, s, a, b):
     assert result == list(map(inc, range(20)))
 
 
-def test_submit_after_failed_worker(loop):
-    with cluster() as (s, [a, b]):
-        with Executor(('127.0.0.1', s['port']), loop=loop) as e:
-            L = e.map(inc, range(10))
-            wait(L)
-            a['proc'].terminate()
-            total = e.submit(sum, L)
-            assert total.result() == sum(map(inc, range(10)))
-
-
-def test_gather_after_failed_worker(loop):
-    with cluster() as (s, [a, b]):
-        with Executor(('127.0.0.1', s['port']), loop=loop) as e:
-            L = e.map(inc, range(10))
-            wait(L)
-            a['proc'].terminate()
-            result = e.gather(L)
-            assert result == list(map(inc, range(10)))
-
-
-@slow
-def test_gather_then_submit_after_failed_workers(loop):
-    with cluster(nworkers=4) as (s, [w, x, y, z]):
-        with Executor(('127.0.0.1', s['port']), loop=loop) as e:
-            L = e.map(inc, range(20))
-            wait(L)
-            w['proc'].terminate()
-            total = e.submit(sum, L)
-            wait([total])
-
-            (_, port) = first(e.scheduler.who_has[total.key])
-            for d in [x, y, z]:
-                if d['port'] == port:
-                    d['proc'].terminate()
-
-            result = e.gather([total])
-            assert result == [sum(map(inc, range(20)))]
-
-
 @gen_cluster(ncores=[('127.0.0.1', 1)], executor=True)
 def test_errors_dont_block(e, s, w):
     L = [e.submit(inc, 1),
@@ -1127,111 +1064,6 @@ def test_traceback_sync(loop):
             assert tb is None
 
 
-@gen_cluster(Worker=Nanny, executor=True)
-def test_restart(e, s, a, b):
-    assert s.ncores == {a.worker_address: 1, b.worker_address: 2}
-
-    x = e.submit(inc, 1)
-    y = e.submit(inc, x)
-    z = e.submit(div, 1, 0)
-    yield y._result()
-
-    assert set(s.who_has) == {x.key, y.key}
-
-    f = yield e._restart()
-    assert f is e
-
-    assert len(s.stacks) == 2
-    assert len(s.processing) == 2
-
-    assert not s.who_has
-
-    assert x.cancelled()
-    assert y.cancelled()
-    assert z.cancelled()
-    assert z.key not in s.exceptions
-
-    assert not s.who_wants
-    assert not s.wants_what
-
-
-@gen_cluster(Worker=Nanny, executor=True)
-def test_restart_cleared(e, s, a, b):
-    x = 2 * delayed(1) + 1
-    f = e.compute(x)
-    yield _wait([f])
-    assert s.released
-
-    yield e._restart()
-
-    for coll in [s.tasks, s.dependencies, s.dependents, s.waiting,
-            s.waiting_data, s.who_has, s.restrictions, s.loose_restrictions,
-            s.released, s.priority, s.exceptions, s.who_wants,
-            s.exceptions_blame]:
-        assert not coll
-
-
-def test_restart_sync_no_center(loop):
-    with cluster(nanny=True) as (s, [a, b]):
-        with Executor(('127.0.0.1', s['port']), loop=loop) as e:
-            x = e.submit(inc, 1)
-            e.restart()
-            assert x.cancelled()
-            y = e.submit(inc, 2)
-            assert y.result() == 3
-            assert len(e.ncores()) == 2
-
-
-def test_restart_sync(loop):
-    with cluster(nanny=True) as (s, [a, b]):
-        with Executor(('127.0.0.1', s['port']), loop=loop) as e:
-            x = e.submit(div, 1, 2)
-            x.result()
-
-            assert sync(loop, e.scheduler.who_has)
-            e.restart()
-            assert not sync(loop, e.scheduler.who_has)
-            assert x.cancelled()
-            assert len(e.ncores()) == 2
-
-            with pytest.raises(CancelledError):
-                x.result()
-
-            y = e.submit(div, 1, 3)
-            assert y.result() == 1 / 3
-
-
-def test_restart_fast(loop):
-    with cluster(nanny=True) as (s, [a, b]):
-        with Executor(('127.0.0.1', s['port']), loop=loop) as e:
-            L = e.map(sleep, range(10))
-
-            start = time()
-            e.restart()
-            assert time() - start < 5
-            assert len(e.ncores()) == 2
-
-            assert all(x.status == 'cancelled' for x in L)
-
-            x = e.submit(inc, 1)
-            assert x.result() == 2
-
-
-@gen_cluster(Worker=Nanny, executor=True)
-def test_fast_kill(e, s, a, b):
-    L = e.map(sleep, range(10))
-
-    start = time()
-    yield e._restart()
-    assert time() - start < 5
-
-    assert all(x.status == 'cancelled' for x in L)
-
-    x = e.submit(inc, 1)
-    result = yield x._result()
-    assert result == 2
-
-
 @gen_cluster(executor=True)
 def test_upload_file(e, s, a, b):
     def g():
@@ -1307,29 +1139,6 @@ def test_multiple_executors(s, a, b):
     yield b._shutdown()
 
 
-@gen_cluster(Worker=Nanny)
-def test_multiple_executors_restart(s, a, b):
-    e1 = Executor((s.ip, s.port), start=False)
-    yield e1._start()
-    e2 = Executor((s.ip, s.port), start=False)
-    yield e2._start()
-
-    x = e1.submit(inc, 1)
-    y = e2.submit(inc, 2)
-    xx = yield x._result()
-    yy = yield y._result()
-    assert xx == 2
-    assert yy == 3
-
-    yield e1._restart()
-
-    assert x.cancelled()
-    assert y.cancelled()
-
-    yield e1._shutdown(fast=True)
-    yield e2._shutdown(fast=True)
-
-
 @gen_cluster(executor=True)
 def test_async_compute(e, s, a, b):
     from dask.delayed import delayed
@@ -1522,40 +1331,6 @@ def test_map_on_futures_with_kwargs(e, s, a, b):
     assert result == 100 + 1 + 200
 
 
-@gen_cluster(Worker=Nanny, timeout=60, executor=True)
-def test_failed_worker_without_warning(e, s, a, b):
-    L = e.map(inc, range(10))
-    yield _wait(L)
-
-    a.process.terminate()
-    start = time()
-    while not a.process.is_alive():
-        yield gen.sleep(0.01)
-        assert time() - start < 10
-
-    yield gen.sleep(0.5)
-
-    start = time()
-    while len(s.ncores) < 2:
-        yield gen.sleep(0.01)
-        assert time() - start < 10
-
-    yield _wait(L)
-
-    L2 = e.map(inc, range(10, 20))
-    yield _wait(L2)
-    assert all(len(keys) > 0 for keys in s.has_what.values())
-    ncores2 = s.ncores.copy()
-
-    yield e._restart()
-
-    L = e.map(inc, range(10))
-    yield _wait(L)
-    assert all(len(keys) > 0 for keys in s.has_what.values())
-
-    assert not (set(ncores2) & set(s.ncores))  # no overlap
-
-
 class BadlySerializedObject(object):
     def __getstate__(self):
         return 1
@@ -1651,7 +1426,7 @@ def test_forget_complex(e, s, A, B):
     assert set(s.tasks) == {f.key for f in [ab,ac,cd,acab]}
 
     s.client_releases_keys(keys=[b.key], client=e.id)
-    assert set(s.tasks) == {f.key for f in [ab,ac,cd,acab]}
+    assert set(s.tasks) == {f.key for f in [ac,cd,acab]}
 
     s.client_releases_keys(keys=[acab.key], client=e.id)
     assert set(s.tasks) == {f.key for f in [ac,cd]}
@@ -1787,32 +1562,6 @@ def test_multi_executor(s, a, b):
     assert not s.tasks
 
 
-@gen_cluster(executor=True, timeout=60)
-def test_broken_worker_during_computation(e, s, a, b):
-    n = Nanny(s.ip, s.port, ncores=2, loop=s.loop)
-    n.start(0)
-
-    start = time()
-    while len(s.ncores) < 3:
-        yield gen.sleep(0.01)
-        assert time() < start + 5
-
-    L = e.map(inc, range(256))
-    for i in range(8):
-        L = e.map(add, *zip(*partition_all(2, L)))
-
-    from random import random
-    yield gen.sleep(random() / 2)
-    n.process.terminate()
-    yield gen.sleep(random() / 2)
-    n.process.terminate()
-
-    result = yield e._gather(L)
-    assert isinstance(result[0], int)
-
-    yield n._close()
-
-
 @gen_cluster()
 def test_cleanup_after_broken_executor_connection(s, a, b):
     def f(ip, port):
@@ -1949,6 +1698,16 @@ def test__cancel(e, s, a, b):
     s.validate_state()
 
 
+@gen_cluster(executor=True)
+def test__cancel_tuple_key(e, s, a, b):
+    x = e.submit(inc, 1, key=('x', 0, 1))
+
+    result = yield x._result()
+    yield e._cancel(x)
+    with pytest.raises(CancelledError):
+        yield x._result()
+
+
 @gen_cluster()
 def test__cancel_multi_client(s, a, b):
     e = Executor((s.ip, s.port), start=False)
@@ -2680,18 +2439,6 @@ def test_workers_register_indirect_data(e, s, a, b):
     s.validate_state()
 
 
-@pytest.mark.skipif(not sys.platform.startswith('linux'),
-                    reason="Need 127.0.0.2 to mean localhost")
-@gen_cluster(executor=True, ncores=[('127.0.0.1', 2), ('127.0.0.2', 2)],
-        timeout=20)
-def test_work_stealing(e, s, a, b):
-    [x] = yield e._scatter([1])
-    futures = e.map(slowadd, range(50), [x] * 50)
-    yield _wait(futures)
-    assert len(a.data) > 10
-    assert len(b.data) > 10
-
-
 @gen_cluster(executor=True)
 def test_submit_on_cancelled_future(e, s, a, b):
     x = e.submit(inc, 1)
@@ -2935,92 +2682,6 @@ def test_scheduler_saturates_cores_random(e, s, a, b):
             yield gen.sleep(0.01)
 
 
-@gen_cluster(executor=True, ncores=[('127.0.0.1', 1)] * 2)
-def test_dont_steal_expensive_data_fast_computation(e, s, a, b):
-    np = pytest.importorskip('numpy')
-    x = e.submit(np.arange, 1000000, workers=a.address)
-    yield _wait([x])
-    future = e.submit(np.sum, [1], workers=a.address)  # learn that sum is fast
-    yield _wait([future])
-
-    cheap = [e.submit(np.sum, x, pure=False, workers=a.address,
-                      allow_other_workers=True) for i in range(10)]
-    yield _wait(cheap)
-    assert len(b.data) == 0
-    assert len(a.data) == 12
-
-
-@gen_cluster(executor=True, ncores=[('127.0.0.1', 1)] * 2)
-def test_steal_cheap_data_slow_computation(e, s, a, b):
-    x = e.submit(slowinc, 100, delay=0.1)  # learn that slowinc is slow
-    yield _wait([x])
-
-    futures = e.map(slowinc, range(10), delay=0.01, workers=a.address,
-                    allow_other_workers=True)
-    yield _wait(futures)
-    assert abs(len(a.data) - len(b.data)) < 3
-
-
-@gen_cluster(executor=True, ncores=[('127.0.0.1', 1)] * 2)
-def test_steal_expensive_data_slow_computation(e, s, a, b):
-    np = pytest.importorskip('numpy')
-
-    x = e.submit(slowinc, 100, delay=0.1, workers=a.address)
-    yield _wait([x])  # learn that slowinc is slow
-
-    x = e.submit(np.arange, 1000000, workers=a.address)  # put expensive data
-    yield _wait([x])
-
-    slow = [e.submit(slowinc, x, delay=0.1, pure=False) for i in range(4)]
-    yield _wait([slow])
-
-    assert b.data  # not empty
-
-
-@gen_cluster(executor=True, ncores=[('127.0.0.1', 1)] * 10)
-def test_worksteal_many_thieves(e, s, *workers):
-    x = e.submit(slowinc, -1, delay=0.1)
-    yield x._result()
-
-    xs = e.map(slowinc, [x] * 100, pure=False, delay=0.01)
-
-    yield _wait(xs)
-
-    for w, keys in s.has_what.items():
-        assert 2 < len(keys) < 50
-
-
-@gen_cluster(executor=True, ncores=[('127.0.0.1', 1)] * 2)
-def test_dont_steal_unknown_functions(e, s, a, b):
-    futures = e.map(inc, [1, 2], workers=a.address, allow_other_workers=True)
-    yield _wait(futures)
-    assert len(a.data) == 2
-    assert len(b.data) == 0
-
-
-@gen_cluster(executor=True, ncores=[('127.0.0.1', 1)] * 2)
-def test_eventually_steal_unknown_functions(e, s, a, b):
-    futures = e.map(slowinc, range(10), delay=0.1,  workers=a.address,
-                    allow_other_workers=True)
-    yield _wait(futures)
-    assert len(a.data) >= 3
-    assert len(b.data) >= 3
-
-
-@gen_cluster(executor=True, ncores=[('127.0.0.1', 1)] * 4, timeout=None)
-def test_cancel_stress(e, s, *workers):
-    da = pytest.importorskip('dask.array')
-    x = da.random.random((40, 40), chunks=(1, 1))
-    x = e.persist(x)
-    yield _wait([x])
-    y = (x.sum(axis=0) + x.sum(axis=1) + 1).std()
-    for i in range(5):
-        f = e.compute(y)
-        while len(s.waiting) > (len(y.dask) - len(x.dask)) / 2:
-            yield gen.sleep(0.01)
-        yield e._cancel(f)
-
-
 @gen_cluster(executor=True, ncores=[('127.0.0.1', 1)] * 4)
 def test_cancel_clears_processing(e, s, *workers):
     da = pytest.importorskip('dask.array')
@@ -3036,19 +2697,6 @@ def test_cancel_clears_processing(e, s, *workers):
         yield gen.sleep(0.01)
     s.validate_state()
 
-def test_cancel_stress_sync(loop):
-    da = pytest.importorskip('dask.array')
-    x = da.random.random((40, 40), chunks=(1, 1))
-    with cluster() as (s, [a, b]):
-        with Executor(('127.0.0.1', s['port']), loop=loop) as e:
-            x = e.persist(x)
-            y = (x.sum(axis=0) + x.sum(axis=1) + 1).std()
-            wait(x)
-            for i in range(5):
-                f = e.compute(y)
-                sleep(1)
-                e.cancel(f)
-
 
 def test_default_get(loop):
     with cluster() as (s, [a, b]):
@@ -3138,6 +2786,7 @@ def test_get_foo(e, s, a, b):
     assert c == {futures[0].key: list(s.who_has[futures[0].key])}
 
 
+@slow
 @gen_cluster(executor=True, Worker=Nanny)
 def test_bad_tasks_fail(e, s, a, b):
     f = e.submit(sys.exit, 1)
@@ -3222,6 +2871,7 @@ def test_get_returns_early(e, s, a, b):
     assert x.key in s.tasks
 
 
+@slow
 @gen_cluster(Worker=Nanny, executor=True)
 def test_Executor_clears_references_after_restart(e, s, a, b):
     x = e.submit(inc, 1)
@@ -3236,16 +2886,6 @@ def test_Executor_clears_references_after_restart(e, s, a, b):
 
     assert key not in e.refcount
 
-@gen_cluster(Worker=Nanny, executor=True)
-def test_forgotten_futures_dont_clean_up_new_futures(e, s, a, b):
-    x = e.submit(inc, 1)
-    yield e._restart()
-    y = e.submit(inc, 1)
-    del x
-    import gc; gc.collect()
-    yield gen.sleep(0.1)
-    yield y._result()
-
 
 def test_get_stops_work_after_error(loop):
     loop2 = IOLoop()
@@ -3282,86 +2922,6 @@ def test_as_completed_list(loop):
             assert set(e.gather(seq2)) == {1, 2, 3, 4, 5}
 
 
-@pytest.mark.ipython
-def test_start_ipython(loop):
-    from jupyter_client import BlockingKernelClient
-    from ipykernel.kernelapp import IPKernelApp
-    from IPython.core.interactiveshell import InteractiveShell
-
-    with cluster(1) as (s, [a]):
-        with Executor(('127.0.0.1', s['port']), loop=loop) as e:
-            info_dict = e.start_ipython()
-            info = first(info_dict.values())
-            key = info.pop('key')
-            kc = BlockingKernelClient(**info)
-            kc.session.key = key
-            kc.start_channels()
-            msg_id = kc.execute("worker")
-            reply = kc.get_shell_msg(timeout=10)
-            kc.stop_channels()
-
-@pytest.mark.ipython
-def test_start_ipython_magic(loop):
-    ip = mock.Mock()
-    with cluster() as (s, [a, b]):
-        with mock.patch('IPython.get_ipython', lambda : ip), Executor(('127.0.0.1', s['port']), loop=loop) as e:
-            workers = list(e.ncores())[:2]
-            names = [ 'magic%i' % i for i in range(len(workers)) ]
-            e.start_ipython(workers, magic_names=names)
-    assert ip.register_magic_function.call_count == 4
-    expected = [
-        {'magic_kind': 'line', 'magic_name': 'magic0'},
-        {'magic_kind': 'cell', 'magic_name': 'magic0'},
-        {'magic_kind': 'line', 'magic_name': 'magic1'},
-        {'magic_kind': 'cell', 'magic_name': 'magic1'},
-    ]
-    call_kwargs_list = [ kwargs for (args, kwargs) in ip.register_magic_function.call_args_list ]
-    assert call_kwargs_list == expected
-
-
-@pytest.mark.ipython
-def test_start_ipython_qtconsole(loop):
-    Popen = mock.Mock()
-    with cluster() as (s, [a, b]):
-        with mock.patch('distributed._ipython_utils.Popen', Popen), Executor(('127.0.0.1', s['port']), loop=loop) as e:
-            worker = first(e.ncores())
-            e.start_ipython(worker, qtconsole=True)
-            e.start_ipython(worker, qtconsole=True, qtconsole_args=['--debug'])
-    assert Popen.call_count == 2
-    (cmd,), kwargs = Popen.call_args_list[0]
-    assert cmd[:3] == [ 'jupyter', 'qtconsole', '--existing' ]
-    (cmd,), kwargs = Popen.call_args_list[1]
-    assert cmd[-1:] == [ '--debug' ]
-
-
-@gen_cluster(ncores=[], executor=True, timeout=None)
-def test_stress_creation_and_deletion(e, s):
-    # Assertions are handled by the validate mechanism in the scheduler
-    s.allowed_failures = 100000
-    da = pytest.importorskip('dask.array')
-
-    x = da.random.random(size=(2000, 2000), chunks=(100, 100))
-    y = (x + 1).T + (x * 2) - x.mean(axis=1)
-
-    z = e.persist(y)
-
-    @gen.coroutine
-    def create_and_destroy_worker(delay):
-        start = time()
-        while time() < start + 10:
-            n = Nanny(s.ip, s.port, ncores=2, loop=s.loop)
-            n.start(0)
-
-            yield gen.sleep(delay)
-
-            yield n._close()
-            print("Killed nanny")
-
-    yield gen.with_timeout(timedelta(minutes=1),
-                          All([create_and_destroy_worker(0.1 * i) for i in
-                              range(10)]))
-
-
 @gen_test()
 def test_status():
     s = Scheduler()
@@ -3413,13 +2973,88 @@ def test_scatter_raises_if_no_workers(e, s):
         yield e._scatter([1])
 
 
+@gen_test()
+def test_synchronize_worker_data():
+    s = Scheduler(synchronize_worker_interval=50)
+    s.start(0)
+    a = Worker(s.ip, s.port, name='alice')
+    yield a._start()
+
+    a.data['x'] = 1
+    response = yield s.synchronize_worker_data()
+
+    assert response == {a.address: {'extra': ['x'], 'missing': []}}
+    assert not a.data
+
+    yield a._close()
+    s.stop()
+
+
+@gen_test()
+def test_synchronize_worker_data_race_condition():
+    s = Scheduler(synchronize_worker_interval=50)
+    s.start(0)
+    a = Worker(s.ip, s.port, name='alice')
+    yield a._start()
+
+    a.data['x'] = 1
+    s.loop.add_callback(s.synchronize_worker_data)
+    yield gen.sleep(0.020)
+    s.has_what[a.address].add('x')
+    s.who_has['x'] = {a.address}
+
+    yield gen.sleep(0.100)
+    assert a.data == {'x': 1}
+
+    yield a._close()
+    s.stop()
+
+
+@gen_test()
+def test_synchronize_worker_data_callback():
+    s = Scheduler(synchronize_worker_interval=50)
+    s.start(0)
+    a = Worker(s.ip, s.port, name='alice')
+    yield a._start()
+
+    a.data['x'] = 1
+
+    yield gen.sleep(0.200)
+
+    assert not a.data
+
+    yield a._close()
+    s.stop()
+
+
+@gen_cluster(executor=True)
+def test_synchronize_missing_data_on_one_worker(e, s, a, b):
+    s.synchronize_worker_interval = 10
+    np = pytest.importorskip('numpy')
+    [f] = yield e._scatter([1], broadcast=True)
+    assert a.data and b.data
+
+    del a.data[f.key]
+
+    yield s.synchronize_worker_data()
+
+    assert s.task_state[f.key] == 'memory'
+    assert b.data
+    # assert set(s.has_what[b.address]) == set(a.data)
+
+
 from distributed.utils_test import popen
+@slow
 def test_reconnect(loop):
     w = Worker('127.0.0.1', 9393, loop=loop)
     w.start()
     with popen(['dask-scheduler', '--port', '9393', '--no-bokeh']) as s:
         e = Executor('localhost:9393', loop=loop)
-        assert len(e.ncores()) == 1
+        start = time()
+        while  len(e.ncores()) != 1:
+            sleep(0.1)
+            assert time() < start + 3
+
         x = e.submit(inc, 1)
         assert x.result() == 2
 
@@ -3498,3 +3133,187 @@ def test_open_close_many_workers(loop, worker, count, repeat):
 
     after = proc.num_fds()
     assert before >= after
+
+
+@gen_cluster(executor=False, timeout=None)
+def test_idempotence(s, a, b):
+    e = Executor((s.ip, s.port), start=False)
+    yield e._start()
+    f = Executor((s.ip, s.port), start=False)
+    yield f._start()
+
+    # Submit
+    x = e.submit(inc, 1)
+    yield x._result()
+    log = list(s.transition_log)
+
+    len_single_submit = len(log)  # see last assert
+
+    y = f.submit(inc, 1)
+    assert x.key == y.key
+    yield y._result()
+    yield gen.sleep(0.1)
+    log2 = list(s.transition_log)
+    assert log == log2
+
+    # Error
+    a = e.submit(div, 1, 0)
+    yield _wait(a)
+    assert a.status == 'error'
+    log = list(s.transition_log)
+
+    b = f.submit(div, 1, 0)
+    assert a.key == b.key
+    yield _wait(b)
+    yield gen.sleep(0.1)
+    log2 = list(s.transition_log)
+    assert log == log2
+
+    s.transition_log.clear()
+    # Simultaneous Submit
+    c = e.submit(inc, 2)
+    d = e.submit(inc, 2)
+    yield _wait([c, d])
+
+    assert len(s.transition_log) == len_single_submit
+
+    yield e._shutdown()
+    yield f._shutdown()
+
+
+def test_scheduler_info(loop):
+    with cluster() as (s, [a, b]):
+        with Executor(('127.0.0.1', s['port']), loop=loop) as e:
+            info = e.scheduler_info()
+            assert isinstance(info, dict)
+            assert len(info['workers']) == 2
+
+
+def test_threaded_get_within_distributed(loop):
+    with cluster() as (s, [a, b]):
+        with Executor(('127.0.0.1', s['port']), loop=loop) as e:
+            import dask.multiprocessing
+            for get in [dask.async.get_sync,
+                        dask.multiprocessing.get,
+                        dask.threaded.get]:
+                def f():
+                    return get({'x': (lambda: 1,)}, 'x')
+
+                future = e.submit(f)
+                assert future.result() == 1
+
+@gen_cluster(executor=True)
+def test_lose_scattered_data(e, s, a, b):
+    [x] = yield e._scatter([1], workers=a.address)
+
+    yield a._close()
+    yield gen.sleep(0.1)
+
+    assert x.status == 'cancelled'
+    assert x.key not in s.task_state
+
+
+
+@gen_cluster(executor=True, ncores=[('127.0.0.1', 1)] * 3)
+def test_partially_lose_scattered_data(e, s, a, b, c):
+    [x] = yield e._scatter([1], workers=a.address)
+    yield e._replicate(x, n=2)
+
+    yield a._close()
+    yield gen.sleep(0.1)
+
+    assert x.status == 'finished'
+    assert s.task_state[x.key] == 'memory'
+
+
+@gen_cluster(executor=True)
+def test_scatter_compute_lose(e, s, a, b):
+    [x] = yield e._scatter([[1, 2, 3, 4]], workers=a.address)
+    y = e.submit(inc, 1)
+
+    z = e.submit(slowadd, x, y, delay=0.2)
+    yield gen.sleep(0.1)
+
+    yield a._close()
+
+    assert x.status == 'cancelled'
+    assert y.status == 'finished'
+    assert z.status == 'cancelled'
+
+    with pytest.raises(CancelledError):
+        yield _wait(z)
+
+
+@gen_cluster(executor=True)
+def test_scatter_compute_store_lose(e, s, a, b):
+    """
+    Create irreplacable data on one machine,
+    cause a dependent computation to occur on another and complete
+
+    Kill the machine with the irreplacable data.  What happens to the complete
+    result?  How about after it GCs and tries to come back?
+    """
+    [x] = yield e._scatter([1], workers=a.address)
+    xx = e.submit(inc, x, workers=a.address)
+    y = e.submit(inc, 1)
+
+    z = e.submit(slowadd, xx, y, delay=0.2, workers=b.address)
+    yield _wait(z)
+
+    yield a._close()
+
+    start = time()
+    while x.status == 'finished':
+        yield gen.sleep(0.01)
+        assert time() < start + 2
+
+    # assert xx.status == 'finished'
+    assert y.status == 'finished'
+    assert z.status == 'finished'
+
+    zz = e.submit(inc, z)
+    yield _wait(zz)
+
+    zkey = z.key
+    del z
+
+    start = time()
+    while s.task_state[zkey] != 'released':
+        yield gen.sleep(0.01)
+        assert time() < start + 2
+
+    xxkey = xx.key
+    del xx
+
+    start = time()
+    while (x.key in s.task_state and
+           zkey not in s.task_state and
+           xxkey not in s.task_state):
+        yield gen.sleep(0.01)
+        assert time() < start + 2
+
+
+@gen_cluster(executor=True)
+def test_scatter_compute_store_lose_processing(e, s, a, b):
+    """
+    Create irreplacable data on one machine,
+    cause a dependent computation to occur on another and complete
+
+    Kill the machine with the irreplacable data.  What happens to the complete
+    result?  How about after it GCs and tries to come back?
+    """
+    [x] = yield e._scatter([1], workers=a.address)
+
+    y = e.submit(slowinc, x, delay=0.2)
+    z = e.submit(inc, y)
+    yield gen.sleep(0.1)
+    yield a._close()
+
+    start = time()
+    while x.status == 'finished':
+        yield gen.sleep(0.01)
+        assert time() < start + 2
+
+    assert y.status == 'cancelled'
+    assert z.status == 'cancelled'
+
