@@ -13,6 +13,7 @@ import sys
 from time import time, sleep
 import uuid
 
+import mock
 from toolz import merge
 from tornado import gen
 from tornado.ioloop import IOLoop, TimeoutError
@@ -79,6 +80,25 @@ def loop():
             print(f)
 
 
+@pytest.yield_fixture
+def zmq_ctx():
+    import zmq
+    ctx = zmq.Context.instance()
+    yield ctx
+    ctx.destroy(linger=0)
+
+
+@contextmanager
+def mock_ipython():
+    ip = mock.Mock()
+    ip.user_ns = {}
+    ip.kernel = None
+    get_ip = lambda : ip
+    with mock.patch('IPython.get_ipython', get_ip), \
+            mock.patch('distributed._ipython_utils.get_ipython', get_ip):
+        yield ip
+
+
 def inc(x):
     return x + 1
 
@@ -119,9 +139,9 @@ def randominc(x, scale=1):
     return x + 1
 
 
-def slowadd(x, y):
+def slowadd(x, y, delay=0.02):
     from time import sleep
-    sleep(0.02)
+    sleep(delay)
     return x + y
 
 
@@ -451,6 +471,8 @@ def start_cluster(ncores, loop, Worker=Worker):
     done = s.start(0)
     workers = [Worker(s.ip, s.port, ncores=v, ip=k, name=i, loop=loop)
                 for i, (k, v) in enumerate(ncores)]
+    for w in workers:
+        w.rpc = workers[0].rpc
 
     yield [w._start() for w in workers]
 
