@@ -8,7 +8,7 @@ from multiprocessing.pool import ThreadPool
 import os
 import pkg_resources
 import tempfile
-from threading import current_thread, Thread
+from threading import current_thread
 from time import time
 from timeit import default_timer
 import shutil
@@ -149,6 +149,7 @@ class Worker(Server):
                     'health': self.host_health,
                     'upload_file': self.upload_file,
                     'start_ipython': self.start_ipython,
+                    'keys': self.keys,
                 }
 
         super(Worker, self).__init__(handlers, io_loop=self.loop, **kwargs)
@@ -300,7 +301,7 @@ class Worker(Server):
         local = {k: self.data[k] for k in who_has if k in self.data}
         who_has = {k: v for k, v in who_has.items() if k not in local}
         remote, bad_data = yield gather_from_workers(who_has,
-                permissive=True, rpc=self.rpc, close=False)
+                permissive=True)
         if remote:
             self.data.update(remote)
             yield self.scheduler.add_keys(address=self.address, keys=list(remote))
@@ -461,6 +462,7 @@ class Worker(Server):
     def compute_one(self, data, key=None, function=None, args=None, kwargs=None,
                     report=False, task=None):
         logger.debug("Compute one on %s", key)
+        self.active.add(key)
         diagnostics = dict()
         try:
             start = default_timer()
@@ -473,7 +475,6 @@ class Worker(Server):
             emsg['key'] = key
             raise Return(emsg)
 
-        self.active.add(key)
         # Fill args with data
         args2 = pack_data(args, data)
         kwargs2 = pack_data(kwargs, data)
@@ -617,51 +618,18 @@ class Worker(Server):
     def get_data(self, stream, keys=None):
         return {k: dumps(self.data[k]) for k in keys if k in self.data}
 
-    def _start_ipython(self):
-        from IPython import get_ipython
-        if get_ipython() is not None:
-            raise RuntimeError("Cannot start IPython, it's already running.")
-
-        from zmq.eventloop.ioloop import ZMQIOLoop
-        from ipykernel.kernelapp import IPKernelApp
-        # save the global IOLoop instance
-        # since IPython relies on it, but we are going to put it in a thread.
-        save_inst = IOLoop.instance()
-        IOLoop.clear_instance()
-        zmq_loop = ZMQIOLoop()
-        zmq_loop.install()
-
-        # start IPython, disabling its signal handlers that won't work due to running in a thread:
-        app = self._ipython_kernel = IPKernelApp.instance(log=logger)
-        # Don't connect to the history database
-        app.config.HistoryManager.hist_file = ':memory:'
-        # listen on all interfaces, so remote clients can connect:
-        app.ip = self.ip
-        app.init_signal = lambda : None
-        app.initialize([])
-        app.kernel.pre_handler_hook = lambda : None
-        app.kernel.post_handler_hook = lambda : None
-        app.kernel.start()
-
-        # save self in the IPython namespace as 'worker'
-        app.kernel.shell.user_ns['worker'] = self
-
-        # start IPython's IOLoop in a thread
-        zmq_loop_thread = Thread(target=zmq_loop.start)
-        zmq_loop_thread.start()
-
-        # put the global IOLoop instance back:
-        IOLoop.clear_instance()
-        save_inst.install()
-        return app
-
     def start_ipython(self, stream):
         """Start an IPython kernel
 
         Returns Jupyter connection info dictionary.
         """
+        from ._ipython_utils import start_ipython
         if self._ipython_kernel is None:
-            self._ipython_kernel = self._start_ipython()
+            self._ipython_kernel = start_ipython(
+                ip=self.ip,
+                ns={'worker': self},
+                log=logger,
+            )
         return self._ipython_kernel.get_connection_info()
 
     def upload_file(self, stream, filename=None, data=None, load=True):
@@ -705,7 +673,7 @@ class Worker(Server):
             mem = psutil.virtual_memory()
             d.update({'cpu': psutil.cpu_percent(),
                       'memory': mem.total,
-                      'memory-percent': mem.percent})
+                      'memory_percent': mem.percent})
 
             net_io = psutil.net_io_counters()
             if self._last_net_io:
@@ -734,6 +702,9 @@ class Worker(Server):
             pass
         return d
 
+    def keys(self, stream=None):
+        return list(self.data)
+
 
 job_counter = [0]
 
